#!/bin/sh

#####
# External resources needed by this script.
#####
WSDL=../server/docs/AgencyIntegrationServices-V1.2.wsdl
# http://atws.grants.gov/wsdl/agency/AgencyIntegrationServices-V1.2.wsdl

AXIS_HOME=c:/axis-1_3

#####
# External libraries needed
# XML related and JSSE libraries needed when using JDK 1.3
#####
JAVAMAIL=C:/javamail-1.3.3_01/mail.jar
JAF=C:/jaf-1.0.2/activation.jar
XMLAPI=c:/xerces-2_6_2/xmlParserAPIs.jar
XERCESIMPL=c:/xerces-2_6_2/xercesImpl.jar
JSSE=c:/jsse1.0.3_03/lib/jsse.jar

#####
# Properties for this script.
#####
LIB=${AXIS_HOME}/lib
CURRDIR=`pwd`
SERVERLIB=../server/lib
CLIENTLIB=../client/lib
STAGEDIR=../stage
GENDIR=${STAGEDIR}/axisGen
AXISSRC=src
SRC=${GENDIR}/${AXISSRC}
CLASSES=${GENDIR}/classes
JARFILE=axisGenWSDL2Java.jar
APIDOC=${GENDIR}/apidoc
JAVADOCTITLE='Agency-WSDL2Java-classes'

#####
# Tell JDK 1.3 to use the JSSE package in jsse.jar, will not need this with 1.4+.
#####
JAVAOPTS=-Djava.protocol.handler.pkgs=com.sun.net.ssl.internal.www.protocol

#####
# skeletonDeploy: Deploy skeleton (true) or implementation (false) in deploy.wsdd.
#                 Default is false.  Assumes --server-side.
# all:            Generate code for all elements, even unreferenced ones.
# output:         Output directory for emitted files.
# server-side:    Emit server-side bindings for web service.
# testCase:       Emit JUnit testcase class for web service.
# May also add "--Debug --verbose" for more info onto screen.
#####
GENOPTS="--skeletonDeploy false --all --output ${SRC}"

#####
# Grab all the jars from the Axis lib directory and form a classpath.
#####
MYCP=`find ${LIB} -name '*.jar' -type f | sort | paste -d';' -s -`

#####
# We need to add these jars to the classpath when using JDK 1.3.
# Will not need this with 1.4+.
#####
MYCP="${MYCP};${XMLAPI};${XERCESIMPL};${JSSE};${JAVAMAIL};${JAF}"

#####
# Create the staging directory if it doesn't exist.
#####
if [ ! -d ${STAGEDIR} ]
then
   echo Creating ${STAGEDIR}
   mkdir ${STAGEDIR}
fi

#####
# Create the generation directory if it doesn't exist.
#####
if [ ! -d ${GENDIR} ]
then
   echo Creating ${GENDIR}
   mkdir ${GENDIR}
fi

#####
# Remove contents of src directory if it exist already or else create it.
#####
if [ -d ${SRC} ]
then
   cd ${SRC}
   FILECOUNT=`find . -type f -name '*.java' | grep -c '.*'`
   if [ ${FILECOUNT} -gt 0 ]
   then
      echo Removing contents of ${SRC}
      rm -r *
   fi
   cd ${CURRDIR}
else
   echo Creating ${SRC}
   mkdir ${SRC}
fi

#####
# Call WSDL2Java.
#####
echo Generating Java files with ${WSDL}
java -cp ${MYCP} ${JAVAOPTS} org.apache.axis.wsdl.WSDL2Java ${GENOPTS} ${WSDL}

if [ $? -ne 0 ]
then
   echo Error occurred in WSDL2Java generation
   exit $?
fi

#####
# Remove classes directory if exist already or else create it.
#####
if [ -d ${CLASSES} ] 
then
   cd ${CLASSES}
   CLASSESCOUNT=`find . -type f | grep -c '.*'`
   if [ ${CLASSESCOUNT} -gt 0 ]
   then
      echo Removing everything from ${CLASSES} 
      rm -r *
   fi
   cd ${CURRDIR}
else
   echo Creating directory ${CLASSES}
   mkdir ${CLASSES}
fi

#####
# Since we need to edit the generated Binding Implementation class,
# we need to rename it so the compiler doesn't compile it which gives us
# a chance to implement it in our code.
#####
BINDINGIMPL=`find ${SRC} -name 'AgencyIntegrationSoapBindingImpl.java'`
BINDINGIMPLCOUNT=`find ${SRC} -name 'AgencyIntegrationSoapBindingImpl.java' | grep -c 'AgencyIntegrationSoapBindingImpl.java'`
if [ ${BINDINGIMPLCOUNT} -eq 1 ] && [ -n ${BINDINGIMPL} ] && [ -f ${BINDINGIMPL} ] 
then
   echo Renaming AgencyIntegrationSoapBindingImpl.java with .generated extension
   mv ${BINDINGIMPL} ${BINDINGIMPL}.generated
fi

#####
# Get all the java files we plan to compile.
#####
FILES=`find ${SRC} -name '*.java' -type f`

#####
# Call javac.
#####
echo Compiling code
javac -deprecation -classpath ${MYCP} -sourcepath ${SRC} -d ${CLASSES} -g:lines,vars,source ${FILES}

#####
# If the compilation was successful, then jar it up and place source
# files in the jar.
#####
if [ $? -eq 0 ]
then
   if [ -f ${SERVERLIB}/${JARFILE} ]
   then
      echo Removing ${SERVERLIB}/${JARFILE}
      rm ${SERVERLIB}/${JARFILE}
   fi
   echo Creating ${SERVERLIB}/${JARFILE}
   jar -cf ${SERVERLIB}/${JARFILE} -C ${CLASSES} .
   jar -uf ${SERVERLIB}/${JARFILE} -C ${GENDIR} ${AXISSRC}
   echo Copying ${SERVERLIB}/${JARFILE} to ${CLIENTLIB}
   cp ${SERVERLIB}/${JARFILE} ${CLIENTLIB}/.
   cd ${CURRDIR}
fi

#####
# Remove apidoc directory if exist already or else create it.
#####
if [ -d ${APIDOC} ]
then
   cd ${APIDOC}
   FILECOUNT=`find . -type f | grep -c '.*'`
   if [ ${FILECOUNT} -gt 0 ]
   then
      echo Removing everything from ${APIDOC}
      rm -r *
   fi
   cd ${CURRDIR}
else
   echo Creating directory ${APIDOC}
   mkdir ${APIDOC}
fi

#####
# Javadoc the source files.
#####
echo Javadocing source
javadoc -use -private -splitindex -version -author -sourcepath ${SRC} -classpath ${MYCP} -d ${APIDOC} -windowtitle ${JAVADOCTITLE} ${FILES} > ${STAGEDIR}/javadoc.log

