/*
 * Created on Jun 1, 2005
 * Unpublished Work, Copyright, 2004 Northrop Grumman
 */
package gov.grants.apply.dao;

import gov.grants.apply.dao.xmlDAO.XmlDAOFactory;
import gov.grants.apply.util.PropertiesLoader;

import org.apache.log4j.Logger;

/**
 * @author Grants.gov WebServices Team
 *
 * An abstract factory for creating DAO objects that access agency grant information.
 */
public abstract class AgencyDAOFactory
{
   private static final Logger LOG = Logger.getLogger(AgencyDAOFactory.class);
   private static final String DAO_FACTORY_CLASS = "DAO_FACTORY_CLASS";
   private static final String APPLICATION_PROPERTIES = "application.properties";
   private final static AgencyDAOFactory _factory;
   
   static
   {
      AgencyDAOFactory factory = null;
      String factoryName = PropertiesLoader.getValue(APPLICATION_PROPERTIES, DAO_FACTORY_CLASS);
      if (factoryName != null)
      {
         try
         {
            Class factoryClass = Class.forName(factoryName);
            if (AgencyDAOFactory.class.isAssignableFrom(factoryClass))
            {
               factory = (AgencyDAOFactory) factoryClass.newInstance();
            }
            else
            {
               LOG.error("DAO factory MUST be of type " + AgencyDAOFactory.class.getName());
            }
         }
         catch (InstantiationException e)
         {
            LOG.error("Could not instantiate DAO factory class.", e);
         }
         catch (IllegalAccessException e)
         {
            LOG.error("Could not instantiate DAO factory class.", e);
         }
         catch (ClassNotFoundException e)
         {
            LOG.error("Could not instantiate DAO factory class.", e);
         }
      }
      if (factory == null)
      {
         factory = new XmlDAOFactory();
      }
      _factory = factory;
      if (LOG.isDebugEnabled())
      {
         LOG.debug("Using DAO factory: " + _factory.getClass().getName());
      }
   }
   
   /**
    * @return The configured DAO factory.
    */
   public static final AgencyDAOFactory getFactory()
   {
      return _factory;
   }
   
   /**
    * @return The DAO for Applications
    */
   public abstract IApplicationDAO createApplicationDAO();
}
