/*
 * Created on Jun 3, 2005
 *
 * Unpublished Work, Copyright, 2005 Northrop Grumman
 */
package gov.grants.apply.ws.production;

import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationListRequestApplicationFilter;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationListRequestApplicationFilterFilter;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationListResponse;
import gov.grants.apply.util.AxisUtils;
import gov.grants.apply.util.ProcessorUtils;
import gov.grants.apply.ws.IGetApplicationListBean;
import gov.grants.apply.ws.IGetApplicationListProcessor;
import gov.grants.apply.ws.ProcessorException;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

import com.trw.InFlowSuiteCustomization.Grants_gov.CustomerDefinedClasses.GGWSWrapper;
import com.trw.InFlowSuiteCustomization.Grants_gov.CustomerDefinedClasses.wswrapper.ApplicationListInfo;

/**
 * @author Grants.gov WebServices Team
 *
 */
public final class GetApplicationListProdModeProcessor
   implements IGetApplicationListProcessor
{
   private static final int OPPORTUNITY_NUM_INDEX = 0;
   private static final int COMPETITION_ID_INDEX = 1;
   private static final int CFDA_NUM_INDEX = 2;
   private static final int TRACKING_NUM_INDEX = 3;
   private static final int RECEIVED_DATE_INDEX = 4;
   private static final int STATUS_INDEX = 5;
   private static final Logger LOG = Logger.getLogger(GetApplicationListProdModeProcessor.class);
   
   /* (non-Javadoc)
    * @see gov.grants.apply.ws.IGetApplicationListProcessor#process(gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationListRequest)
    */
   public GetApplicationListResponse process(GetApplicationListRequestApplicationFilter[] filters)
         throws ProcessorException
   {
      LOG.debug("Processing request.");
      List filterList = new ArrayList();
      String value = null;
      String[] pair = null;
      
      for (int i = 0; filters != null && i < filters.length && filters[i] != null; i++)
      {
         pair = new String[2];
         value = filters[i].getFilterValue();
         if (value != null && !"".equals(value))
         {
            if (filters[i].getFilter() == GetApplicationListRequestApplicationFilterFilter.CFDANumber)
            {
               pair[0] = GGWSWrapper.FILTER_CFDA;
               pair[1] = value;
            }
            else if (filters[i].getFilter() == GetApplicationListRequestApplicationFilterFilter.OpportunityID)
            {
               pair[0] = GGWSWrapper.FILTER_OPPNUM;
               pair[1] = value;
            }
            else if (filters[i].getFilter() == GetApplicationListRequestApplicationFilterFilter.Status)
            {
               pair[0] = GGWSWrapper.FILTER_STATUS;
               pair[1] = value;
            }
            filterList.add(pair);
         }
      }
      List applications = null;
      try
      {
         applications = GGWSWrapper.getApplicationList(GGWSWrapper.getID(AxisUtils.getCurrentHttpRequest()), filterList);
      }
      catch (Exception e)
      {
         LOG.error(GGWSWrapper.getExceptionMessage(e.getMessage()), e);
         throw new ProcessorException("GGWSWrapper failed when retrieving application list.", e);
      }
      
      if (applications == null)
      {
         return ProcessorUtils.prepareGetApplicationListResponse(null);
      }
      
      Iterator iter = applications.iterator();
      int size = applications.size();
      IGetApplicationListBean[] appInfos = new IGetApplicationListBean[size];
      Object val = null;
      ApplicationListInfo info = null;
      List stringList = null;
      SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'hh:mm:ss");
      for (int i = 0; i < size && iter.hasNext(); i++)
      {
         stringList = (List) iter.next();
         info = new ApplicationListInfo();
         val = stringList.get(OPPORTUNITY_NUM_INDEX);
         if (val != null)
         {
            info.setOpportunityNumber(val.toString());
         }
         val = stringList.get(COMPETITION_ID_INDEX);
         if (val != null)
         {
            info.setCompetitionId(val.toString());
         }
         val = stringList.get(CFDA_NUM_INDEX);
         if (val != null)
         {
            info.setCfdaNumber(val.toString());
         }
         val = stringList.get(TRACKING_NUM_INDEX);
         if (val != null)
         {
            info.setTrackingNumber(val.toString());
         }
         val = stringList.get(RECEIVED_DATE_INDEX);
         if (val != null)
         {
            try
            {
               if (LOG.isDebugEnabled())
               {
                  LOG.debug("Received Date: " + val.toString());
               }
               int index = val.toString().lastIndexOf('-');
               String formattedDate = val.toString().substring(0, index);
               info.setReceivedDate(dateFormat.parse(formattedDate));
            }
            catch (ParseException e)
            {
               throw new ProcessorException(e);
            }
         }
         val = stringList.get(STATUS_INDEX);
         if (val != null)
         {
            info.setStatus(val.toString());
         }
         appInfos[i] = new ApplicationInfoFromGGWSWrapper(info);
      }
      LOG.debug("Sending response.");
      return ProcessorUtils.prepareGetApplicationListResponse(appInfos);
   }

}
