/*
 * Created on Jun 6, 2005
 *
 * Unpublished Work, Copyright, 2005 Northrop Grumman
 */
package gov.grants.apply.ws.production;

import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.EncodingTypeEnum;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationRequest;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationResponse;
import gov.grants.apply.util.AxisUtils;
import gov.grants.apply.util.ByteArrayDataSource;
import gov.grants.apply.util.ProcessorUtils;
import gov.grants.apply.ws.GetApplicationBean;
import gov.grants.apply.ws.IGetApplicationProcessor;
import gov.grants.apply.ws.ProcessorException;

import java.io.ByteArrayInputStream;
import java.util.Iterator;
import java.util.Map;

import com.trw.InFlowSuiteCustomization.Grants_gov.CustomerDefinedClasses.GGWSWrapper;

/**
 * @author Grants.gov WebServices Team
 *
 * Process the GetApplication web service request using the GGWSWrapper interface into
 * the Grants.gov server.
 */
public final class GetApplicationProdModeProcessor
   implements IGetApplicationProcessor
{

   private static final int MIMETYPE_INDEX = 1;
   private static final int CONTENT_ID_INDEX = 0;
   /**
    * The GGWSWrapper class returns a weird Map object back as the result of the
    * getApplication invocation.  The grant application XML is indexed with the
    * GGWSWrapper.GRANT_APPLICATION_XML_KEY key.  The rest of the map holds a series of
    * attachments.  The weirdness is in how the attachments are represented: the key to
    * an attachment is a String array that holds the contentID and mimeType information
    * as its zeroth and first elements, respectively, the value gotten by this key is a
    * byte array that is the attachment file.  You cannot ask the Map for an attachment
    * unless you form a String array key with the contentID and mimeType (hopefully, the
    * Strings are externalized).  Since we won't have that information, you are forced
    * to iterate thru the set of Map Entries.  Basically, we're going to treat the Map
    * as a List and the key/value pairs as an Object that holds two Strings and a byte
    * array.
    * 
    * @param request The GetApplication web service request
    * @return A null reference since we are setting the SOAP response manually.
    */
   public GetApplicationResponse process(GetApplicationRequest request)
         throws ProcessorException
   {
      String trackingNum = request.getGrants_govTrackingNumber();
      EncodingTypeEnum encodingType = request.getEncodingType();
      String encoding = DEFAULT_ENCODING_TYPE;
      
      if (trackingNum == null)
      {
         throw new ProcessorException("Tracking number is null.");
      }
      
      byte[] appXml = null;
      if (encodingType != null)
      {
         encoding = encodingType.getValue();
      }
      Map app = null;
      
      try
      {
         app = GGWSWrapper.getApplication(GGWSWrapper.getID(AxisUtils.getCurrentHttpRequest()), trackingNum);
      }
      catch (Exception e) // forced to catch all Exceptions
      {
         throw new ProcessorException(e);
      }
      if (app != null) // GGWSWrapper returns null if application could not be retrieved.
      {
         appXml = (byte[]) app.get(GGWSWrapper.GRANT_APPLICATION_XML_KEY);
         if (appXml == null || appXml.length == 0)
         {
            throw new ProcessorException("The grant application was found, but the application XML could not be retrieved or had zero bytes.");
         }
         ByteArrayInputStream stream = new ByteArrayInputStream(appXml);
         Iterator entries = app.entrySet().iterator();
         Map.Entry entry = null;
         Object key = null;
         Object value = null;
         String[] info = null;
         byte[] data = null;
         while (entries.hasNext())
         {
            entry = (Map.Entry) entries.next();
            key = entry.getKey();
            value = entry.getValue();
            if (GGWSWrapper.GRANT_APPLICATION_XML_KEY.equals(key))
            {
               continue;
            }
            if (key instanceof String[] && value instanceof byte[]) // is there another way of checking, besides instanceof?
            {
               info = (String[]) key;
               data = (byte[]) value;
               if (info.length == 2)
               {
                  ProcessorUtils.addAttachmentToResponse(new ByteArrayDataSource(info[MIMETYPE_INDEX], info[CONTENT_ID_INDEX], data), info[CONTENT_ID_INDEX], encoding);
               }
            }
         }
         GetApplicationBean bean = new GetApplicationBean();
         bean.setDataSource(new ByteArrayDataSource(XML_CONTENT_TYPE, trackingNum + DOT_XML_EXT, appXml));
         return ProcessorUtils.prepareGetApplicationResponse(bean);
      }
      return new GetApplicationResponse(); // return an empty response
   }

}
