/*
 * Created on Jun 1, 2005
 *
 * Unpublished Work, Copyright, 2004 Northrop Grumman
 */
package gov.grants.apply.dao;

import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GrantsGovApplicationStatusType;

import javax.activation.DataSource;

/**
 * @author Grants.gov WebServices Team
 *
 * This is a value object for the IApplicationDAO class.
 * @see gov.grants.apply.dao.IApplicationDAO
 */
public final class Application
{
   private SF424Header _header = null;
   private SF424Footer _footer = null;
   private GrantsGovApplicationStatusType _status = null;
   private DataSource _xmlSource = null;
   
   /**
    * Set the XML data source.
    * @param xmlSource the DataSource that accesses this application's XML data.
    */
   public void setXmlSource(DataSource xmlSource)
   {
      _xmlSource = xmlSource;
   }
   
   /**
    * @return The DataSource that accesses this application's XML data.
    */
   public DataSource getXmlSource()
   {
      return _xmlSource;
   }
   
   /**
    * @return Returns the footer of this application.
    */
   public SF424Footer getFooter()
   {
      return _footer;
   }
   /**
    * @param footer The footer of this application.
    */
   public void setFooter(SF424Footer footer)
   {
      _footer = footer;
   }
   /**
    * @return Returns the header of this application.
    */
   public SF424Header getHeader()
   {
      return _header;
   }
   /**
    * @param header The header of this application.
    */
   public void setHeader(SF424Header header)
   {
      _header = header;
   }
   
   /**
    * Set this property using the constants given in the gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GrantsGovApplicationStatusType class.
    * The constants are named "value1" to "value8," their definitions are the following:
    * <ul>
    *    <li>value1 = "Receiving"</li>
    *    <li>value2 = "Received"</li>
    *    <li>value3 = "Processing"</li>
    *    <li>value4 = "Validated"</li>
    *    <li>value5 = "Rejected with Errors"</li>
    *    <li>value6 = "Download Preparation"</li>
    *    <li>value7 = "Received by Agency"</li>
    *    <li>value8 = "Agency Tracking Number Assigned"</li>
    * </ul>
    * @param status The status of this application
    * @see gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GrantsGovApplicationStatusType
    */
   public void setApplicationStatus(GrantsGovApplicationStatusType status)
   {
      _status = status;
   }
   
   /**
    * @return The status of this application
    */
   public GrantsGovApplicationStatusType getApplicationStatus()
   {
      return _status;
   }
}
