/*
 * Created on Jun 9, 2005
 *
 * Unpublished Work, Copyright, 2005 Northrop Grumman
 */
package gov.grants.apply.util;

import java.io.PrintStream;
import java.io.PrintWriter;

/**
 * @author Grants.gov WebServices Team
 *
 * A temporary replacement for jdk14's throwable class.  Temporary until we upgrade to 1.4
 */
public class NestedException
   extends Exception
{
   private static final String SEPARATOR = ": ";
   private static final String NESTED_EXCEPTION = "Nested Exception: ";
   private Throwable _cause = null;
   private String _message = null;

   public NestedException()
   {
      super();
   }
   
   /**
    * Create the exception with an error message.
    * @param message Error message
    */
   public NestedException(String message)
   {
      this(message, null);
   }
   
   /**
    * Create the exception with a nested cause.
    * @param cause The nested cause of the error.
    */
   public NestedException(Throwable cause)
   {
      this(null, cause);
   }
   
   /**
    * Create the exception with an error message and a nested cause.
    * @param message Error message
    * @param cause The nested cause of the error.
    */
   public NestedException(String message, Throwable cause)
   {
      super();
      _message = message;
      _cause = cause;
   }

   /**
    * Sets the cause of the error.
    * @param cause The nested cause of the error.
    */
   protected void setCause(Throwable cause)
   {
      _cause = cause;
   }

   /**
    * Sets the error message.
    * @param message The error message.
    */
   protected void setMessage(String message)
   {
      _message = message;
   }
   
   /**
    * Prints the stack trace to stderr.  If there is a message or a cause for the error,
    * the message will also be printed and the stack trace from the cause of the error
    * is printed.
    */
   public void printStackTrace()
   {
      if (_message != null)
      {
         System.err.println(getClass().getName() + SEPARATOR + _message);
      }
      super.printStackTrace();
      if (_cause != null)
      {
         System.err.print(NESTED_EXCEPTION);
         _cause.printStackTrace();
      }
   }
   
   /**
    * Prints the stack trace to the stream.  If there is a message or a cause for the
    * error, the message will also be printed and the stack trace from the cause of the
    * error is printed.
    */
   public void printStrackTrace(PrintStream s)
   {
      if (_message != null)
      {
         s.println(getClass().getName() + SEPARATOR + _message);
      }
      super.printStackTrace(s);
      if (_cause != null)
      {
         s.print(NESTED_EXCEPTION);
         _cause.printStackTrace();
      }
   }

   /**
    * Prints the stack trace to the writer.  If there is a message or a cause for the
    * error, the message will also be printed and the stack trace from the cause of the
    * error is printed.
    */
   public void printStackTrace(PrintWriter w)
   {
      if (_message != null)
      {
         w.println(getClass().getName() + SEPARATOR + _message);
      }
      super.printStackTrace(w);
      if (_cause != null)
      {
         w.print(NESTED_EXCEPTION);
         _cause.printStackTrace(w);
      }
   }
   
   /**
    * @return the error message set.
    */
   public String getMessage()
   {
      return _message;
   }
   
   /**
    * @return The String representation of this exception.
    */
   public String toString()
   {
      if (_message != null)
      {
         return getClass().getName() + SEPARATOR + _message;
      }
      return getClass().getName();
   }
}
