/*
 * Created on May 13, 2005
 *
 * Unpublished Work, Copyright, 2005 Northrop Grumman
 */
package gov.grants.apply.util;

import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.ApplicationInformationType;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.AssignAgencyTrackingNumberResponse;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.ConfirmApplicationDeliveryResponse;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationListResponse;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationResponse;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationZipResponse;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.Item;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.ResponseMessage;
import gov.grants.apply.ws.GetApplicationBean;
import gov.grants.apply.ws.GetApplicationZipBean;
import gov.grants.apply.ws.IGetApplicationListBean;
import gov.grants.apply.ws.ProcessorException;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.soap.AttachmentPart;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPPart;
import javax.xml.transform.stream.StreamSource;

import org.apache.axis.Message;
import org.apache.axis.MessageContext;
import org.apache.axis.attachments.Attachments;
import org.apache.axis.attachments.DynamicContentDataHandler;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * @author Grants.gov WebServices Team
 *
 * Common utilities shared by processors from different modes.
 */
public final class ProcessorUtils
{
   private static final Logger LOG = Logger.getLogger(ProcessorUtils.class);
   private static final byte[] GET_APPLICATION_SOAP_START = "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\"><soapenv:Body><GetApplicationResponse xmlns=\"http://apply.grants.gov/WebServices/AgencyIntegrationServices-V1.0\">".getBytes();
   private static final byte[] GET_APPLICATION_SOAP_END = "</GetApplicationResponse></soapenv:Body></soapenv:Envelope>".getBytes();
   
   /**
    * Creates an Axis response object with the given argument.  If no apps were found
    * (i.e. the apps argument is null), then a response with an empty list of
    * ApplicationInformationType objects will be returned.
    * @param apps A bean hold all necessary information to create the response object.
    * @return The Axis response object.
    */
   public static GetApplicationListResponse prepareGetApplicationListResponse(IGetApplicationListBean[] apps)
   {
      ApplicationInformationType[] list = null;
      if (apps == null)
      {
         list = new ApplicationInformationType[0];  
      }
      else
      {
         list = new ApplicationInformationType[apps.length];
         String value = null;
         for (int i = 0; i < apps.length; i++)
         {
            ApplicationInformationType appInfo = new ApplicationInformationType();
            value = apps[i].getCfdaNumber();
            if (value != null)
            {
               appInfo.setCFDANumber(value);
            }
            value = apps[i].getCompetitionId();
            if (value != null)
            {
               appInfo.setCompetitionID(value);
            }
            appInfo.setGrants_govTrackingNumber(apps[i].getTrackingNumber());
            value = apps[i].getOpportunityNumber();
            if (value != null)
            {
               appInfo.setOpportunityID(value);
            }
            appInfo.setReceivedDateTime(apps[i].getReceivedDate());
            appInfo.setGrants_GovApplicationStatus(apps[i].getStatus());
            list[i] = appInfo;
         }
      }
      
      GetApplicationListResponse response = new GetApplicationListResponse();
      response.setApplicationInformation(list);
      response.setAvailableApplicationNumber(list.length);
      return response;
   }
   
   /**
    * Creates the Axis generated response object for GetApplication WebService call.
    * @param bean Information for the response.
    * @throws ProcessorException
    */
   public static GetApplicationResponse prepareGetApplicationResponse(GetApplicationBean bean)
      throws ProcessorException
   {
      GetApplicationResponse response = null;
      InputStream xmlStream = null;
      DataSource appXmlSource = bean.getDataSource();
      
      if (appXmlSource == null)
      {
         throw new ProcessorException("The application XML data source is null, was the GetApplication service unable to get the application?");
      }
      try
      {
         xmlStream = appXmlSource.getInputStream();
      }
      catch (IOException e)
      {
         throw new ProcessorException("Unable to get input stream from the application XML data source.", e);
      }
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setNamespaceAware(true);
      factory.setValidating(false);
      DocumentBuilder builder = null;
      
      try
      {
         builder = factory.newDocumentBuilder();
      }
      catch (ParserConfigurationException e)
      {
         throw new ProcessorException("Could not create a document builder with GrantApplication XML.", e);
      }
     
      try
      {
         Document doc = builder.parse(xmlStream);
         response = new GetApplicationResponse();
         response.setSubmittedApplication(doc);
      }
      catch (SAXException e)
      {
         throw new ProcessorException("Could not read the GrantApplication XML.", e);
      }
      catch (IOException e)
      {
         throw new ProcessorException("Could not read the GrantApplication XML.", e);
      }
      return response;
   }
   
   /**
    * Not using this code right now.
    * @param xmlStream
    * @throws IOException
    * @throws SOAPException
    */
   private static void manuallyInsertResponseMessage(InputStream xmlStream)
      throws IOException, SOAPException
   {
      MessageContext axisContext = MessageContext.getCurrentContext();
      Message response = axisContext.getResponseMessage();
      ByteArrayOutputStream buf = new ByteArrayOutputStream();
      ByteArrayInputStream soapMessage = null;
      
      try
      {
         buf.write(GET_APPLICATION_SOAP_START, 0, GET_APPLICATION_SOAP_START.length);
         FileUtil.pipe(buf, xmlStream);
         buf.write(GET_APPLICATION_SOAP_END, 0, GET_APPLICATION_SOAP_END.length);
         soapMessage = new ByteArrayInputStream(buf.toByteArray());
      }
      finally
      {
         try
         {
            buf.close();
         }
         catch (IOException e)
         {
            LOG.warn("Could not close ByteArrayOutputStream that is holding the GetApplication response SOAP XML.", e);
         }
         try
         {
            xmlStream.close();
         }
         catch (IOException e)
         {
            LOG.error("Unable to close input stream for application XML.", e);
         }
      }
      if (response == null)
      {
         response = new Message(soapMessage, true);
      }
      else
      {
         SOAPPart soapPart = response.getSOAPPart();
         soapPart.setContent(new StreamSource(soapMessage));
      }
      axisContext.setResponseMessage(response);
   }
   
   /**
    * Creates Axis response object with a message and the Grants.gov tracking number.
    * @param message The success or failure message.
    * @param ggTrackNum The Grants.gov tracking number of the application.
    * @return The Axix response object.
    */
   public static AssignAgencyTrackingNumberResponse prepareAssignAgencyTrackingNumberResponse(ResponseMessage message, String ggTrackNum)
   {
      AssignAgencyTrackingNumberResponse response = new AssignAgencyTrackingNumberResponse();
      response.setGrants_govTrackingNumber(ggTrackNum);
      response.setResponseMessage(message);
      return response;
   }
   
   /**
    * Creates Axis response object with a message and the Grants.gov tracking number.
    * @param message The success or failure message.
    * @param ggTrackNum The Grants.gov tracking number of the application.
    * @return The Axix response object.
    */
   public static ConfirmApplicationDeliveryResponse prepareConfirmAppDeliveryResponse(ResponseMessage message, String ggTrackNum)
   {
      ConfirmApplicationDeliveryResponse response = new ConfirmApplicationDeliveryResponse();
      response.setGrants_govTrackingNumber(ggTrackNum);
      response.setResponseMessage(message);
      return response;
   }
   
   /**
    * Creates the Axis response object with the given bean argument.  Also adds the zip
    * to the SOAP attachment part of the message.
    * @param bean Contains all required information to form the Axis response and the data source for the zip file.
    * @return The Axis response.
    */
   public static GetApplicationZipResponse prepareGetApplicationZipResponse(GetApplicationZipBean bean)
   {
      GetApplicationZipResponse response = new GetApplicationZipResponse();
      DataSource zip = bean.getDataSource();
      if (zip != null)
      {
         Item item = new Item();
         item.setFileLocation(bean.getFileLocation());
         item.setFileName(bean.getFileName());
         item.setMimeType(bean.getMimeType());
         item.setPurpose(bean.getPurpose());
         Item[] items = {item};     
         response.setManifest(items);
         addAttachmentToResponse(zip, bean.getFileName(), bean.getEncodingType());
      }
      return response;
   }
   
   /**
    * Adds and DataSource object as an attachment to the SOAP response.
    * @param dataSource The attachment.
    * @param contentId The content ID of the attachment.
    * @throws java.lang.UnsupportedOperationException if this deployment of Axis is not configured for attachment support (mostly likely because the deployment is missing activation.jar and mail.jar).
    */
   public static void addAttachmentToResponse(DataSource dataSource, String contentId, String clientEncodingType)
   {
      if (dataSource != null)
      {
         Message response = MessageContext.getCurrentContext().getResponseMessage();
         Attachments attachments = response.getAttachmentsImpl();
         /*
          * If this returns null, then NO ATTACHMENT SUPPORT EXISTS in this
          * configuration of Axis, and no attachment operations may be performed.
          */
         if (attachments == null)
         {
            LOG.error("Message.getAttachmentImpl() returned null, so no attachment support is configured in this installation of Axis.");
            throw new UnsupportedOperationException("Message.getAttachmentImpl() returned null, so no attachment support is configured in this installation of Axis.");
         }
         String appEncodingType = PropertiesLoader.getValue(Constants.APPLICATION_PROPERTIES_FILENAME, Constants.ENCODING_TYPE_PROP_KEY);
         DataHandler dataHandler = null;
         if (Constants.DIME_ENCODING_TYPE.equals(appEncodingType) && Constants.DIME_ENCODING_TYPE.equals(clientEncodingType))
         {
            LOG.debug("Use DIME encoding for attachments.");
            DynamicContentDataHandler dyDataHandler = new DynamicContentDataHandler(dataSource);
            dyDataHandler.setChunkSize(Math.max(PropertiesLoader.getIntValue(Constants.APPLICATION_PROPERTIES_FILENAME, Constants.DIME_MAX_CHUNK_SIZE_PROP_KEY), 10) * 1024 * 1024);
            dataHandler = dyDataHandler;
            attachments.setSendType(Attachments.SEND_TYPE_DIME);
         }
         else
         {
            if (LOG.isDebugEnabled())
            {
               LOG.debug("Not using DIME encoding for attachments because the web application was configured to use " + appEncodingType + " and the client request wanted to use " + clientEncodingType);
            }
            dataHandler = new DataHandler(dataSource);
         }
         AttachmentPart part = response.createAttachmentPart(dataHandler);
         part.setMimeHeader(Constants.CONTENT_DISPOSITION_MIME_HEADER, "attachment; filename=\"" + dataSource.getName() + "\"");
         part.setContentId(contentId);
         response.addAttachmentPart(part);
         MessageContext.getCurrentContext().setResponseMessage(response);
      }
   }
}
