/*
 * Created on Jun 1, 2005
 *
 * Unpublished Work, Copyright, 2004 Northrop Grumman
 */
package gov.grants.apply.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Properties;

import org.apache.log4j.Logger;

public final class PropertiesLoader
{
   private static final Logger LOG = Logger.getLogger(PropertiesLoader.class);
   private static final HashMap _cache = new HashMap();
   
   public static String list(String file)
   {
      Properties prop = _getProperties(file);
      if (prop == null)
      {
         return "No such properties file: " + file;
      }
      StringWriter sw = new StringWriter();
      PrintWriter pw = new PrintWriter(sw);
      prop.list(pw);
      pw.flush();
      sw.flush();
      String value = sw.toString();
      try
      {
         pw.close();
         sw.close();
      }
      catch (IOException e)
      {
         LOG.warn("Could not close PrintWriter and/or StringWriter instances.", e);
      }
      return value;
   }
   
   public static URL getURLValue(String file, String key)
      throws MalformedURLException
   {
      String value = getValue(file, key);
      if (value == null)
      {
         return null;
      }
      return new URL(value);
   }
   
   /**
    * This method retrieves the property in the specified properties file with the
    * specified key and converts the value to the boolean.  If the key or file does not
    * exist, false is returned by default.
    * @param file The name of the properties file that is in the classpath.
    * @param key The name of the property within the properties file.
    * @return The value of the property as a boolean, or false if not found.
    */
   public static boolean getBooleanValue(String file, String key)
   {
      String value = getValue(file, key);
      if (value == null)
      {
         return false;
      }
      return Boolean.valueOf(value).booleanValue();
   }
   
   /**
    * Retrieves the property and returns it as an int.  If the property was not found,
    * zero is returned.
    * @param file The name of the properties file that is in the classpath.
    * @param key The name of the property that is in the properties file.
    * @return The value of the property as an int.
    */
   public static int getIntValue(String file, String key)
   {
      String value = getValue(file, key);
      if (value == null)
      {
         return 0;
      }
      return Integer.parseInt(value);
   }
   
   /**
    * This method retrieves the property in the specified properties file with the
    * specified key.  If the key or file does not exist, null is returned by default.
    * @param file The name of the properties file that is in the classpath.
    * @param key The name of the property within the properties file.
    * @return The value of the property as a boolean, or null if not found.
    */
   public static String getValue(String file, String key)
   {
      Object value = null;
      Properties props = _getProperties(file);
      if (props != null)
      {
         value = props.get(key);
      }
      else if (props == null && LOG.isInfoEnabled())
      {
         LOG.info("No such properties file "+file+" in classpath");
      }

      if (value == null && (value = System.getProperty(key)) == null)
      {
         if (LOG.isInfoEnabled())
         {
            LOG.info("No such property "+key+" in properties file "+file+" or within the System.properties");
         }
         return null;
      }
      return value.toString();
   }
   
   private synchronized static Properties _getProperties(String file)
   {
      if (file == null || "SYSTEM".equals(file))
      {
         return System.getProperties();
      }
      
      Properties props = (Properties) _cache.get(file);
      
      if (props != null)
      {
         return props;
      }
      
      InputStream stream = PropertiesLoader.class.getClassLoader().getResourceAsStream(file);
      if (stream != null)
      {
         props = new Properties();
         try
         {
            props.load(stream);
         }
         catch (IOException e)
         {
            LOG.warn("Unable to load properties file.", e);
            return null;
         }
         _cache.put(file, props);
         return props;
      }
      return null;
   }
}
