/*
 * Created on Jun 7, 2005
 *
 * Unpublished Work, Copyright, 2005 Northrop Grumman
 */
package gov.grants.apply.ws;

import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.ItemFileLocation;
import gov.grants.apply.util.ByteArrayDataSource;

import javax.activation.DataSource;

import org.apache.axis.types.URI;
import org.apache.axis.types.URI.MalformedURIException;
import org.apache.log4j.Logger;

/**
 * @author Grants.gov WebServices Team
 *
 * A transfer object holding resultant data from the GetApplicationZip web service
 * invocation that will be used to create the appropriate SOAP response.
 */
public final class GetApplicationZipBean
{
   private static final Logger LOG = Logger.getLogger(GetApplicationZipBean.class);
   private static final String PURPOSE = "GrantApplication Zip";
   private static final String CONTENT_TYPE = "application/zip";
   
   private DataSource _dataSource = null;
   private String _filename = null;
   private ItemFileLocation _fileLocation = null;
   private String _encodingType = null;

   public final void setEncodingType(String encodingType)
   {
      _encodingType = encodingType;
   }
   
   public final String getEncodingType()
   {
      return _encodingType;
   }
   
   /**
    * Creates the ItemFileLocation object.
    * @param fileLocation The file location string.
    * @throws MalformedURIException If the fileLocation argument is not a proper URI.
    */
   public final void setFileLocation(String fileLocation)
      throws MalformedURIException
   {
      URI href = new URI(fileLocation);
      _fileLocation = new ItemFileLocation();
      _fileLocation.setHref(href);
   }

   /**
    * Creates the DataSource object out of the byte array.
    * @param zipBytes The zip in bytes.
    * @param filename The name of the zip file.
    */
   public final void setZipBytes(byte[] zipBytes, String filename)
   {
      _filename = filename;
      if (zipBytes != null && zipBytes.length > 0)
      {
         setDataSource(new ByteArrayDataSource(CONTENT_TYPE, _filename, zipBytes));
      }
   }

   /**
    * Sets the zip file DataSource, the file name will be set from DataSource.getName()
    * @param dataSource The zip file.
    * @see javax.activation.DataSource#getName()
    */
   public final void setDataSource(DataSource dataSource)
   {
      _filename = dataSource.getName();
      _dataSource = dataSource;
   }
   
   /**
    * @return The DataSource
    */
   public final DataSource getDataSource()
   {
      return _dataSource;
   }
   
   /**
    * @return The file location
    */
   public final ItemFileLocation getFileLocation()
   {
      return _fileLocation;
   }
   
   /**
    * @return The file name
    */
   public final String getFileName()
   {
      return _filename;
   }
   
   /**
    * @return The constant mime type: "application/zip"
    */
   public final String getMimeType()
   {
      return CONTENT_TYPE;
   }
   
   /**
    * @return The constant purpose string: "GrantApplication Zip"
    */
   public final String getPurpose()
   {
      return PURPOSE;
   }
}
