/*
 * Created on Jun 2, 2005
 *
 * Unpublished Work, Copyright, 2005 Northrop Grumman
 */
package gov.grants.apply.ws;

import gov.grants.apply.util.NestedException;

import org.apache.log4j.Logger;

import com.trw.InFlowSuiteCustomization.Grants_gov.CustomerDefinedClasses.GGWSWrapper;

/**
 * @author Grants.gov WebServices Team
 *
 * A nested exception that uses GGWSWrapper to get server error messages.
 */
public final class ProcessorException
   extends NestedException
{
   private static final Logger LOG = Logger.getLogger(NestedException.class);
   private static final String PAREN_END = ")";
   private static final String PAREN_START = ", (";

   /**
    * Create a ProcessorException with an error message.  Also appends the server message
    * to the custom error message supplied in the arguments.
    * @param message The error message
    */
   public ProcessorException(String message)
   {
      this(message, null);
   }
   
   /**
    * Create a ProcessorException with the original Throwable.
    * @param cause The original Throwable.
    */
   public ProcessorException(Throwable cause)
   {
      this(null, cause);
   }
   
   /**
    * Create a ProcessorException with an error message and the original Throwable.  Also
    * appends the server message to the custom error message supplied in the arguments.
    * @param message The error message.
    * @param cause The original Throwable.
    */
   public ProcessorException(String message, Throwable cause)
   {
      super();
      _log(message, cause);
      String desc = null;
      if (cause != null && ProcessorFactory.isProductionMode())
      {
         desc = GGWSWrapper.getExceptionMessage(cause.getMessage());
      }
      if (desc != null)
      {
         if (message == null)
         {
            setMessage(desc);
         }
         else
         {
            StringBuffer buf = new StringBuffer(message.length() + desc.length() + PAREN_START.length() + PAREN_END.length());
            buf.append(message);
            buf.append(PAREN_START);
            buf.append(desc);
            buf.append(PAREN_END);
            setMessage(buf.toString());
         }
      }
      else if (message != null)
      {
         setMessage(message);
      }
      if (cause != null)
      {
         setCause(cause);
      }
   }
   
   private void _log(String message, Throwable cause)
   {
      if (message == null && cause == null)
      {
         return;
      }
      if (message == null && cause != null)
      {
         LOG.error("", cause);
      }
      if (message != null && cause == null)
      {
         LOG.error(message);
      }
      if (message != null && cause != null)
      {
         LOG.error(message, cause);
      }
   }
}
