/*
 * Created on Jun 6, 2005
 *
 * Unpublished Work, Copyright, 2005 Northrop Grumman
 */
package gov.grants.apply.ws.standAloneMode;

import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.EncodingTypeEnum;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationRequest;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationResponse;
import gov.grants.apply.dao.AgencyDAOFactory;
import gov.grants.apply.dao.Application;
import gov.grants.apply.dao.DAOException;
import gov.grants.apply.dao.GetApplicationListFilter;
import gov.grants.apply.dao.IApplicationDAO;
import gov.grants.apply.util.FileUtil;
import gov.grants.apply.util.ProcessorUtils;
import gov.grants.apply.util.WebAppContext;
import gov.grants.apply.ws.GetApplicationBean;
import gov.grants.apply.ws.IGetApplicationProcessor;
import gov.grants.apply.ws.ProcessorException;

import java.io.File;
import java.io.FileNotFoundException;

import javax.activation.FileDataSource;

import org.apache.log4j.Logger;

/**
 * @author Grants.gov WebServices Team
 *
 * Process the GetApplication web service request using the samples that are installed
 * with the Agency WebService reference implementation.
 */
public final class GetApplicationSAModeProcessor
   implements IGetApplicationProcessor
{
   private static final Logger LOG = Logger.getLogger(GetApplicationSAModeProcessor.class);
   
   /**
    * Process the request.
    * @param request The GetApplication web service request
    * @return A null reference since we are setting the SOAP response manually.
    */
   public GetApplicationResponse process(GetApplicationRequest request)
      throws ProcessorException
   {
      GetApplicationListFilter filter = new GetApplicationListFilter();
      String trackingNum = request.getGrants_govTrackingNumber();
      if (trackingNum == null)
      {
         throw new ProcessorException("Tracking number is null.");
      }
      EncodingTypeEnum encodingType = request.getEncodingType();
      String encoding = DEFAULT_ENCODING_TYPE;
      if (encodingType != null)
      {
         encoding = encodingType.getValue();
      }
      filter.setGrantsGovTrackingNum(trackingNum);
      IApplicationDAO appDao = AgencyDAOFactory.getFactory().createApplicationDAO();
      Application[] apps = null;
      try
      {
         apps = appDao.findApplications(filter);
      }
      catch (DAOException e)
      {
         throw new ProcessorException(e);
      }
      if (apps.length == 0)
      {
         return null;
      }
      GetApplicationBean bean = new GetApplicationBean();

      bean.setDataSource(apps[0].getXmlSource());
      File samplesDirPath = new File(WebAppContext.getInstance().getRealPath("samples"));
      File sampleAttachmentFile = null;
      try
      {
         sampleAttachmentFile = FileUtil.findFileWithExt(samplesDirPath, ".pdf");
      }
      catch (FileNotFoundException e)
      {
         throw new ProcessorException(e);
      }
      ProcessorUtils.addAttachmentToResponse(new FileDataSource(sampleAttachmentFile), sampleAttachmentFile.getName(), encoding);
      return ProcessorUtils.prepareGetApplicationResponse(bean);
   }

}
