/*
 * Created on Jun 7, 2005
 *
 * Unpublished Work, Copyright, 2005 Northrop Grumman
 */
package gov.grants.apply.ws.standAloneMode;

import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.EncodingTypeEnum;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationZipRequest;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationZipResponse;
import gov.grants.apply.dao.AgencyDAOFactory;
import gov.grants.apply.dao.Application;
import gov.grants.apply.dao.DAOException;
import gov.grants.apply.dao.GetApplicationListFilter;
import gov.grants.apply.dao.IApplicationDAO;
import gov.grants.apply.util.FileUtil;
import gov.grants.apply.util.ProcessorUtils;
import gov.grants.apply.util.WebAppContext;
import gov.grants.apply.ws.GetApplicationZipBean;
import gov.grants.apply.ws.IGetApplicationProcessor;
import gov.grants.apply.ws.IGetApplicationZipProcessor;
import gov.grants.apply.ws.ProcessorException;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.zip.Deflater;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.axis.types.URI.MalformedURIException;
import org.apache.log4j.Logger;

/**
 * @author Grants.gov WebServices Team
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public final class GetApplicationZipSAModeProcessor
implements IGetApplicationZipProcessor
{
   public static final Logger LOG = Logger.getLogger(GetApplicationZipSAModeProcessor.class);

   /* (non-Javadoc)
    * @see gov.grants.apply.ws.IGetApplicationZipProcessor#process(gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0._GetApplicationZipRequest)
    */
   public final GetApplicationZipResponse process(GetApplicationZipRequest request)
         throws ProcessorException
   {
      String trackingNum = request.getGrants_govTrackingNumber();
      EncodingTypeEnum encodingType = request.getEncodingType();
      String encoding = IGetApplicationProcessor.DEFAULT_ENCODING_TYPE;
      if (encodingType != null)
      {
         encoding = encodingType.getValue();
      }
      GetApplicationListFilter filter = new GetApplicationListFilter();
      if (trackingNum == null || "".equals(trackingNum))
      {
         throw new ProcessorException("Tracking number is null or blank.");
      }
      filter.setGrantsGovTrackingNum(trackingNum);
      IApplicationDAO dao = AgencyDAOFactory.getFactory().createApplicationDAO();
      Application[] apps = null;
      try
      {
         apps = dao.findApplications(filter);
      }
      catch (DAOException e)
      {
         throw new ProcessorException(e);
      }
      GetApplicationZipBean bean = new GetApplicationZipBean();
      bean.setEncodingType(encoding);
      if (apps == null || apps.length == 0)
      {
         throw new ProcessorException("Application '" + trackingNum + "' could not be found.");
      }
      byte[] zip = zip(apps[0]);
      if (zip != null)
      {
         String name = trackingNum + ".zip";
         try
         {
            bean.setFileLocation("cid:" + name);
         }
         catch (MalformedURIException e)
         {
            throw new ProcessorException(e);
         }
         bean.setZipBytes(zip, name);
      }
      return ProcessorUtils.prepareGetApplicationZipResponse(bean);
   }

   private final byte[] zip(Application app)
   {
      ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
      ZipOutputStream zipStream = new ZipOutputStream(byteStream);
      zipStream.setLevel(Deflater.BEST_SPEED);
      
      try
      {
         LOG.debug("Adding sample application xml to zip.");
         addApplicationXmlToZip(app, zipStream);
         LOG.debug("Adding sample attachment to zip.");
         addAttachmentToZip(zipStream);
         LOG.debug("Done with zipping.");
         zipStream.finish();
      }
      catch (IOException e)
      {
         LOG.warn(e);
         return null;
      }
      return byteStream.toByteArray();
   }
   
   private void addApplicationXmlToZip(Application app, ZipOutputStream zipStream)
      throws IOException
   {
      InputStream inStream = new BufferedInputStream(app.getXmlSource().getInputStream());
      OutputStream outStream = new BufferedOutputStream(zipStream);
      zipStream.putNextEntry(new ZipEntry(app.getFooter().getGrantsGovTrackingNumber() + ".xml"));
      FileUtil.pipe(outStream, inStream);
      zipStream.closeEntry();
      inStream.close();    
   }
   
   private void addAttachmentToZip(ZipOutputStream zipStream)
   {
      File samplesDirPath = new File(WebAppContext.getInstance().getRealPath("samples"));
      FileInputStream attachmentStream =null;
      File sampleAttachmentFile = null;
      OutputStream outStream = new BufferedOutputStream(zipStream);
      InputStream inStream = null;
      
      try
      {
         sampleAttachmentFile = FileUtil.findFileWithExt(samplesDirPath, ".pdf");
         attachmentStream = new FileInputStream(sampleAttachmentFile);
         inStream = new BufferedInputStream(attachmentStream);
         zipStream.putNextEntry(new ZipEntry(sampleAttachmentFile.getName()));
         FileUtil.pipe(outStream, inStream);
         zipStream.closeEntry();
      }
      catch (IOException e)
      {
         LOG.error("Could not add attachment file to zip file.", e);
      }
      
      try
      {
         inStream.close();
      }
      catch (IOException e)
      {
         LOG.warn("Could not close attachment file stream.", e);
      }
   }
}
